"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = useItemKeys;

var _debug = _interopRequireDefault(require("../utility/debug.js"));

var _react = require("react");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function useItemKeys(_ref) {
  var getItemId = _ref.getItemId;
  // List items are rendered with `key`s so that React doesn't
  // "reuse" `itemComponent`s in cases when `items` are changed.
  var itemKeyPrefix = (0, _react.useRef)(); // Generates a unique `key` prefix for list item components.

  var generateItemKeyPrefix = (0, _react.useMemo)(function () {
    var counter = 0;

    function getNextCounter() {
      if (counter === Number.MAX_SAFE_INTEGER) {
        counter = 0;
      }

      counter++;
      return counter;
    }

    return function () {
      itemKeyPrefix.current = String(getNextCounter());
    };
  }, [itemKeyPrefix]);
  (0, _react.useMemo)(function () {
    // Generate an initial unique `key` prefix for list item components.
    generateItemKeyPrefix();
  }, []); // If `getItemId()` function is defined, then item `id`s are gonna be the item element `key`s.

  var usesAutogeneratedItemKeys = !getItemId;
  var generateItemKeyPrefixIfNotUsingItemIds = (0, _react.useCallback)(function () {
    if (usesAutogeneratedItemKeys) {
      generateItemKeyPrefix();
      (0, _debug["default"])('React: ~ Item key prefix:', itemKeyPrefix.current);
    }
  }, [usesAutogeneratedItemKeys, generateItemKeyPrefix]);
  /**
   * Returns a `key` for an `item`'s element.
   * @param  {object} item — The item.
   * @param  {number} i — Item's index in `items` list.
   * @return {any}
   */

  var getItemKey = (0, _react.useCallback)(function (item, i) {
    if (getItemId) {
      return getItemId(item);
    }

    return "".concat(itemKeyPrefix.current, ":").concat(i);
  }, [getItemId, itemKeyPrefix]);
  return {
    getItemKey: getItemKey,
    usesAutogeneratedItemKeys: usesAutogeneratedItemKeys,
    updateItemKeysForNewItems: generateItemKeyPrefixIfNotUsingItemIds
  };
}
//# sourceMappingURL=useItemKeys.js.map