"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = useSetItemState;

var _react = require("react");

function useSetItemState(_ref) {
  var initialItemsCount = _ref.initialItemsCount,
      virtualScroller = _ref.virtualScroller;
  // Only compute the initial cache value once.
  var initialCacheValue = (0, _react.useMemo)(function () {
    return new Array(initialItemsCount);
  }, []); // Handler functions cache.

  var cache = (0, _react.useRef)(initialCacheValue); // Caches per-item `setItemState` functions' "references"
  // so that item components don't get re-rendered needlessly.

  var getSetItemState = (0, _react.useCallback)(function (i) {
    if (!cache.current[i]) {
      cache.current[i] = function (itemState) {
        return virtualScroller.setItemState(i, itemState);
      };
    }

    return cache.current[i];
  }, [virtualScroller, cache]);
  return getSetItemState;
}
//# sourceMappingURL=useSetItemState.js.map