"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _ItemNotRenderedError = _interopRequireDefault(require("../ItemNotRenderedError.js"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }

var ItemsContainer = /*#__PURE__*/function () {
  /**
   * Constructs a new "container" from an element.
   * @param {function} getElement
   */
  function ItemsContainer(getElement) {
    _classCallCheck(this, ItemsContainer);

    this.getElement = getElement;
  }
  /**
   * Returns an item element's "top offset", relative to the items `container`'s top edge.
   * @param  {number} renderedElementIndex — An index of an item relative to the "first shown item index". For example, if the list is showing items from index 8 to index 12 then `renderedElementIndex = 0` would mean the item at index `8`.
   * @return {number}
   */


  _createClass(ItemsContainer, [{
    key: "getNthRenderedItemTopOffset",
    value: function getNthRenderedItemTopOffset(renderedElementIndex) {
      var children = this.getElement().children;
      var maxWidth = this.getElement().width;
      var topOffset = this.getElement().paddingTop;
      var rowWidth;
      var rowHeight;
      var startNewRow = true;

      if (renderedElementIndex > children.length - 1) {
        throw new _ItemNotRenderedError["default"]({
          renderedElementIndex: renderedElementIndex,
          renderedElementsCount: children.length
        });
      }

      var i = 0;

      while (i <= renderedElementIndex) {
        if (startNewRow || rowWidth + children[i].width > maxWidth) {
          if (i > 0) {
            topOffset += rowHeight;
            topOffset += children[i].marginTop;
          }

          rowWidth = children[i].width;
          rowHeight = children[i].height;

          if (rowWidth > maxWidth) {
            startNewRow = true;
          } else {
            startNewRow = false;
          }
        } else {
          rowWidth += children[i].width;
          rowHeight = Math.max(rowHeight, children[i].height);
        }

        i++;
      }

      return topOffset;
    }
    /**
     * Returns an item element's height.
     * @param  {number} renderedElementIndex — An index of an item relative to the "first shown item index". For example, if the list is showing items from index 8 to index 12 then `renderedElementIndex = 0` would mean the item at index `8`.
     * @return {number}
     */

  }, {
    key: "getNthRenderedItemHeight",
    value: function getNthRenderedItemHeight(renderedElementIndex) {
      var children = this.getElement().children;

      if (renderedElementIndex > children.length - 1) {
        throw new _ItemNotRenderedError["default"]({
          renderedElementIndex: renderedElementIndex,
          renderedElementsCount: children.length
        });
      }

      return children[renderedElementIndex].height;
    }
    /**
     * Returns items container height.
     * @return {number}
     */

  }, {
    key: "getHeight",
    value: function getHeight() {
      var children = this.getElement().children;
      var maxWidth = this.getElement().width;
      var contentHeight = this.getElement().paddingTop;
      var i = 0;

      while (i < children.length) {
        var rowWidth = 0;
        var rowHeight = 0;

        while (rowWidth <= maxWidth && i < children.length) {
          if (rowWidth === 0 && i > 0) {
            var verticalSpacing = children[i].marginTop;
            contentHeight += verticalSpacing;
          }

          rowWidth += children[i].width;
          rowHeight = Math.max(rowHeight, children[i].height);
          i++;
        }

        contentHeight += rowHeight;
      }

      contentHeight += this.getElement().paddingBottom;
      return contentHeight;
    }
    /**
     * Removes all item elements of an items container.
     */

  }, {
    key: "clear",
    value: function clear() {
      this.getElement().children = [];
    }
  }]);

  return ItemsContainer;
}();

exports["default"] = ItemsContainer;
//# sourceMappingURL=ItemsContainer.js.map