var _excluded = ["as", "items", "itemComponent", "itemComponentProps", "estimatedItemHeight", "getEstimatedItemHeight", "getEstimatedVisibleItemRowsCount", "bypass", "tbody", "preserveScrollPosition", "preserveScrollPositionOnPrependItems", "measureItemsBatchSize", "scrollableContainer", "getScrollableContainer", "getColumnsCount", "getItemId", "className", "onMount", "onItemFirstRender", "onItemInitialRender", "initialScrollPosition", "onScrollPositionChange", "onStateChange", "initialState", "getInitialItemState"];

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

import React, { useRef, useMemo, useLayoutEffect } from 'react';
import PropTypes from 'prop-types';
import useState from './useState.js';
import useVirtualScroller from './useVirtualScroller.js';
import useVirtualScrollerStartStop from './useVirtualScrollerStartStop.js';
import useInstanceMethods from './useInstanceMethods.js';
import useItemKeys from './useItemKeys.js';
import useSetItemState from './useSetItemState.js';
import useOnItemHeightDidChange from './useOnItemHeightDidChange.js';
import useSetNewItemsOnItemsPropertyChange from './useSetNewItemsOnItemsPropertyChange.js';
import useUpdateItemKeysOnItemsChange from './useUpdateItemKeysOnItemsChange.js';
import useClassName from './useClassName.js';
import useStyle from './useStyle.js'; // When `items` property changes:
// * A new `items` property is supplied to the React component.
// * The React component re-renders itself.
// * `useSetNewItemsOnItemsPropertyChange()` hook is run.
// * `useSetNewItemsOnItemsPropertyChange()` hook detects that the `items` property
//   has changed and calls `VirtualScroller.setItems(items)`.
// * `VirtualScroller.setItems(items)` calls `VirtualScroller.setState()`.
// * `VirtualScroller.setState()` calls the `setState()` function.
// * The `setState()` function calls a setter from a `useState()` hook.
// * The React component re-renders itself the second time.

function VirtualScroller(_ref, ref) {
  var AsComponent = _ref.as,
      itemsProperty = _ref.items,
      Component = _ref.itemComponent,
      itemComponentProps = _ref.itemComponentProps,
      estimatedItemHeight = _ref.estimatedItemHeight,
      getEstimatedItemHeight = _ref.getEstimatedItemHeight,
      getEstimatedVisibleItemRowsCount = _ref.getEstimatedVisibleItemRowsCount,
      bypass = _ref.bypass,
      tbody = _ref.tbody,
      preserveScrollPosition = _ref.preserveScrollPosition,
      preserveScrollPositionOnPrependItems = _ref.preserveScrollPositionOnPrependItems,
      measureItemsBatchSize = _ref.measureItemsBatchSize,
      scrollableContainer = _ref.scrollableContainer,
      getScrollableContainer = _ref.getScrollableContainer,
      getColumnsCount = _ref.getColumnsCount,
      getItemId = _ref.getItemId,
      className = _ref.className,
      onMount = _ref.onMount,
      onItemFirstRender = _ref.onItemFirstRender,
      onItemInitialRender = _ref.onItemInitialRender,
      initialScrollPosition = _ref.initialScrollPosition,
      onScrollPositionChange = _ref.onScrollPositionChange,
      onStateChange = _ref.onStateChange,
      initialState = _ref.initialState,
      getInitialItemState = _ref.getInitialItemState,
      rest = _objectWithoutProperties(_ref, _excluded);

  // List items "container" DOM Element reference.
  var container = useRef(); // Create a `VirtualScroller` instance.

  var virtualScroller = useVirtualScroller({
    items: itemsProperty,
    // `estimatedItemHeight` property name is deprecated,
    // use `getEstimatedItemHeight` property instead.
    estimatedItemHeight: estimatedItemHeight,
    getEstimatedItemHeight: getEstimatedItemHeight,
    getEstimatedVisibleItemRowsCount: getEstimatedVisibleItemRowsCount,
    bypass: bypass,
    // bypassBatchSize,
    tbody: tbody,
    onItemInitialRender: onItemInitialRender,
    // `onItemFirstRender(i)` is deprecated, use `onItemInitialRender(item)` instead.
    onItemFirstRender: onItemFirstRender,
    initialScrollPosition: initialScrollPosition,
    onScrollPositionChange: onScrollPositionChange,
    measureItemsBatchSize: measureItemsBatchSize,
    // `scrollableContainer` property is deprecated.
    // Use `getScrollableContainer()` property instead.
    scrollableContainer: scrollableContainer,
    getScrollableContainer: getScrollableContainer,
    getColumnsCount: getColumnsCount,
    getItemId: getItemId,
    AsComponent: AsComponent,
    initialState: initialState,
    getInitialItemState: getInitialItemState,
    onStateChange: onStateChange
  }, {
    container: container
  }); // Only compute the initial state once.

  var _initialState = useMemo(function () {
    return virtualScroller.getInitialState();
  }, []); // Use React's `useState()` hook for managing `VirtualScroller`'s state lifecycle.
  // This way, React will re-render the component on every state update.


  var _useState = useState({
    initialState: _initialState,
    onRender: virtualScroller.onRender,
    itemsProperty: itemsProperty
  }),
      getState = _useState.getState,
      setState = _useState.setState,
      stateToRender = _useState.stateToRender; // Use custom (external) state storage in the `VirtualScroller`.


  useMemo(function () {
    virtualScroller.useState({
      getState: getState,
      setState: setState
    });
  }, []); // Start `VirtualScroller` on mount.
  // Stop `VirtualScroller` on unmount.

  useVirtualScrollerStartStop(virtualScroller); // List items are rendered with `key`s so that React doesn't
  // "reuse" `itemComponent`s in cases when `items` are changed.

  var _useItemKeys = useItemKeys({
    getItemId: getItemId
  }),
      getItemKey = _useItemKeys.getItemKey,
      usesAutogeneratedItemKeys = _useItemKeys.usesAutogeneratedItemKeys,
      updateItemKeysForNewItems = _useItemKeys.updateItemKeysForNewItems; // Cache per-item `setItemState` functions' "references"
  // so that item components don't get re-rendered needlessly.


  var getSetItemState = useSetItemState({
    initialItemsCount: itemsProperty.length,
    virtualScroller: virtualScroller
  }); // Cache per-item `onItemHeightDidChange` functions' "references"
  // so that item components don't get re-rendered needlessly.

  var getOnItemHeightDidChange = useOnItemHeightDidChange({
    initialItemsCount: itemsProperty.length,
    virtualScroller: virtualScroller
  }); // Calls `.setItems()` if `items` property has changed.

  useSetNewItemsOnItemsPropertyChange(itemsProperty, {
    virtualScroller: virtualScroller,
    // `preserveScrollPosition` property name is deprecated,
    // use `preserveScrollPositionOnPrependItems` property instead.
    preserveScrollPosition: preserveScrollPosition,
    preserveScrollPositionOnPrependItems: preserveScrollPositionOnPrependItems
  }); // Updates `key`s if item indexes have changed.

  useUpdateItemKeysOnItemsChange(stateToRender.items, {
    virtualScroller: virtualScroller,
    usesAutogeneratedItemKeys: usesAutogeneratedItemKeys,
    updateItemKeysForNewItems: updateItemKeysForNewItems
  }); // Add instance methods to the React component.

  useInstanceMethods(ref, {
    virtualScroller: virtualScroller
  });
  useLayoutEffect(function () {
    // (deprecated)
    // `onMount()` option is deprecated due to no longer being used.
    // If someone thinks there's a valid use case for it, create an issue.
    if (onMount) {
      onMount();
    }
  }, []); // `willRender()` function is no longer used.
  //
  // // `getSnapshotBeforeUpdate()` is called right before `componentDidUpdate()`.
  // // A hook equivalent/workaround for `getSnapshotBeforeUpdate()`:
  // // https://github.com/facebook/react/issues/15221#issuecomment-583448887
  // //
  // getSnapshotBeforeUpdate(prevProps, prevState) {
  // 	if (this.state !== prevState) {
  // 		this.willRender(this.state, prevState)
  // 	}
  // 	// Returns `null` to avoid React warning:
  // 	// "A snapshot value (or null) must be returned. You have returned undefined".
  // 	return null
  // }

  className = useClassName(className, {
    tbody: tbody
  });
  var style = useStyle({
    tbody: tbody,
    state: stateToRender
  });
  var currentItems = stateToRender.items,
      itemStates = stateToRender.itemStates,
      firstShownItemIndex = stateToRender.firstShownItemIndex,
      lastShownItemIndex = stateToRender.lastShownItemIndex;
  return /*#__PURE__*/React.createElement(AsComponent, _extends({}, rest, {
    ref: container,
    className: className,
    style: style
  }), currentItems.map(function (item, i) {
    if (i >= firstShownItemIndex && i <= lastShownItemIndex) {
      // * Passing `item` as `children` property is legacy and is deprecated.
      //   If version `2.x` is published in some hypothetical future,
      //   the `item` and `itemIndex` properties should be moved below
      //   `{...itemComponentProps}`.
      //
      // * Passing `itemIndex` property is legacy and is deprecated.
      //   The rationale is that setting new `items` on a React component
      //   is an asynchronous operation, so when a user obtains `itemIndex`,
      //   they don't know which `items` list does that index correspond to,
      //   therefore making it useless, or even buggy if used incorreclty.
      //
      // * Passing `onStateChange` property for legacy reasons.
      //   The new property name is `setState`.
      //   The old property name `onStateChange` is deprecated.
      //
      return /*#__PURE__*/React.createElement(Component, _extends({
        item: item,
        itemIndex: i
      }, itemComponentProps, {
        key: getItemKey(item, i),
        state: itemStates && itemStates[i],
        setState: getSetItemState(i),
        onStateChange: getSetItemState(i),
        onHeightChange: getOnItemHeightDidChange(i),
        onHeightDidChange: getOnItemHeightDidChange(i)
      }), item);
    }

    return null;
  }));
}

VirtualScroller = /*#__PURE__*/React.forwardRef(VirtualScroller);
export default VirtualScroller; // `PropTypes.elementType` is available in some version of `prop-types`.
// https://github.com/facebook/prop-types/issues/200

var elementType = PropTypes.elementType || PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.object]);
VirtualScroller.propTypes = {
  as: elementType,
  items: PropTypes.arrayOf(PropTypes.any).isRequired,
  itemComponent: elementType.isRequired,
  itemComponentProps: PropTypes.object,
  // `estimatedItemHeight` property name is deprecated,
  // use `getEstimatedItemHeight` property instead.
  estimatedItemHeight: PropTypes.number,
  getEstimatedItemHeight: PropTypes.func,
  getEstimatedVisibleItemRowsCount: PropTypes.func,
  bypass: PropTypes.bool,
  // bypassBatchSize: PropTypes.number,
  tbody: PropTypes.bool,
  preserveScrollPositionOnPrependItems: PropTypes.bool,
  // `preserveScrollPosition` property name is deprecated,
  // use `preserveScrollPositionOnPrependItems` instead.
  preserveScrollPosition: PropTypes.bool,
  measureItemsBatchSize: PropTypes.number,
  // `scrollableContainer` property is deprecated.
  // Use `getScrollableContainer()` property instead.
  scrollableContainer: PropTypes.any,
  getScrollableContainer: PropTypes.func,
  getColumnsCount: PropTypes.func,
  getItemId: PropTypes.func,
  className: PropTypes.string,
  onMount: PropTypes.func,
  onItemInitialRender: PropTypes.func,
  // `onItemFirstRender(i)` is deprecated, use `onItemInitialRender(item)` instead.
  onItemFirstRender: PropTypes.func,
  initialScrollPosition: PropTypes.number,
  onScrollPositionChange: PropTypes.func,
  onStateChange: PropTypes.func,
  initialState: PropTypes.shape({
    items: PropTypes.arrayOf(PropTypes.object).isRequired,
    itemStates: PropTypes.arrayOf(PropTypes.any).isRequired,
    firstShownItemIndex: PropTypes.number.isRequired,
    lastShownItemIndex: PropTypes.number.isRequired,
    beforeItemsHeight: PropTypes.number.isRequired,
    afterItemsHeight: PropTypes.number.isRequired,
    itemHeights: PropTypes.arrayOf(PropTypes.number).isRequired,
    columnsCount: PropTypes.number,
    verticalSpacing: PropTypes.number
  }),
  getInitialItemState: PropTypes.func
};
VirtualScroller.defaultProps = {
  as: 'div'
};
//# sourceMappingURL=VirtualScroller.js.map