import { useMemo, useRef, useCallback } from 'react';
export default function useOnItemHeightDidChange(_ref) {
  var initialItemsCount = _ref.initialItemsCount,
      virtualScroller = _ref.virtualScroller;
  // Only compute the initial cache value once.
  var initialCacheValue = useMemo(function () {
    return new Array(initialItemsCount);
  }, []); // Handler functions cache.

  var cache = useRef(initialCacheValue); // Caches per-item `onItemHeightDidChange` functions' "references"
  // so that item components don't get re-rendered needlessly.

  var getOnItemHeightDidChange = useCallback(function (i) {
    if (!cache.current[i]) {
      cache.current[i] = function () {
        return virtualScroller.onItemHeightDidChange(i);
      };
    }

    return cache.current[i];
  }, [virtualScroller, cache]);
  return getOnItemHeightDidChange;
}
//# sourceMappingURL=useOnItemHeightDidChange.js.map