import { useMemo, useRef, useCallback } from 'react';
export default function useSetItemState(_ref) {
  var initialItemsCount = _ref.initialItemsCount,
      virtualScroller = _ref.virtualScroller;
  // Only compute the initial cache value once.
  var initialCacheValue = useMemo(function () {
    return new Array(initialItemsCount);
  }, []); // Handler functions cache.

  var cache = useRef(initialCacheValue); // Caches per-item `setItemState` functions' "references"
  // so that item components don't get re-rendered needlessly.

  var getSetItemState = useCallback(function (i) {
    if (!cache.current[i]) {
      cache.current[i] = function (itemState) {
        return virtualScroller.setItemState(i, itemState);
      };
    }

    return cache.current[i];
  }, [virtualScroller, cache]);
  return getSetItemState;
}
//# sourceMappingURL=useSetItemState.js.map