function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }

import VirtualScroller from '../VirtualScroller.js';
import Engine from './Engine.js';
var DEBUG = false;

var TestVirtualScroller = /*#__PURE__*/function () {
  function TestVirtualScroller(_ref) {
    var _this = this;

    var scrollableContainerWidth = _ref.screenWidth,
        scrollableContainerHeight = _ref.screenHeight,
        columnsCount = _ref.columnsCount,
        verticalSpacing = _ref.verticalSpacing,
        items = _ref.items,
        initialState = _ref.state;

    _classCallCheck(this, TestVirtualScroller);

    this.expectedStateUpdates = [];
    var scrollableContainerElement = {
      width: scrollableContainerWidth,
      height: scrollableContainerHeight
    };
    var containerElement = {
      paddingTop: 0,
      paddingBottom: 0,
      width: scrollableContainerElement.width,
      children: []
    };

    this.setScrollableContainerWidth = function (width) {
      scrollableContainerElement.width = width;
      containerElement.width = width;
    };

    this.setScrollableContainerHeight = function (height) {
      scrollableContainerElement.height = height;
    };

    var getItemsContainerElement = function getItemsContainerElement() {
      return containerElement;
    };

    this.__columnsCount = columnsCount;

    var getColumnsCount = function getColumnsCount() {
      return _this.__columnsCount;
    };

    var getItemWidth = function getItemWidth() {
      return scrollableContainerElement.width / getColumnsCount();
    };

    this.hasPausedStateUpdates = undefined;
    this.pausedStateUpdate = undefined;
    this.pausedStateUpdateAction = undefined;

    var onBeforeUpdateState = function onBeforeUpdateState(stateUpdate) {
      if (_this.expectedStateUpdates.length > 0) {
        var expectedStateUpdate = _this.expectedStateUpdates.shift();

        expect(stateUpdate).to.deep.equal(expectedStateUpdate.stateUpdate);

        if (expectedStateUpdate.callback) {
          expectedStateUpdate.callback();
        }
      }
    };

    var render = function render() {
      var _this$virtualScroller = _this.virtualScroller.getState(),
          items = _this$virtualScroller.items,
          beforeItemsHeight = _this$virtualScroller.beforeItemsHeight,
          afterItemsHeight = _this$virtualScroller.afterItemsHeight,
          firstShownItemIndex = _this$virtualScroller.firstShownItemIndex,
          lastShownItemIndex = _this$virtualScroller.lastShownItemIndex;

      console.log('~ Render List ~');
      containerElement.paddingTop = beforeItemsHeight;
      containerElement.paddingBottom = afterItemsHeight;
      containerElement.children = items.slice(firstShownItemIndex, lastShownItemIndex + 1).map(function (item) {
        return {
          width: getItemWidth(),
          height: item.area / getItemWidth(),
          marginTop: verticalSpacing
        };
      });
    };

    this.virtualScroller = new VirtualScroller(getItemsContainerElement, items, {
      scrollableContainer: scrollableContainerElement,
      engine: Engine,
      _waitForScrollingToStop: false,
      getColumnsCount: getColumnsCount,
      state: initialState,
      onStateChange: function onStateChange(state) {
        if (DEBUG) {
          console.log('~ Updated State ~');
          console.log(state);
        }
      }
    });
    var state = this.virtualScroller.getInitialState();
    this.virtualScroller.useState({
      getState: function getState() {
        return state;
      },
      updateState: function updateState(stateUpdate) {
        var stateUpdateAction = function stateUpdateAction(stateUpdate) {
          // Is only used in tests.
          if (onBeforeUpdateState) {
            onBeforeUpdateState(stateUpdate);
          }

          state = _objectSpread(_objectSpread({}, state), stateUpdate);
          render();

          _this.virtualScroller.onRender();
        };

        if (_this.hasPausedStateUpdates) {
          _this.pausedStateUpdate = _objectSpread(_objectSpread({}, _this.pausedStateUpdate), stateUpdate);
          _this.pausedStateUpdateAction = stateUpdateAction;
        } else {
          stateUpdateAction(stateUpdate);
        }
      }
    });
    render();
  }

  _createClass(TestVirtualScroller, [{
    key: "pauseStateUpdates",
    value: function pauseStateUpdates() {
      if (this.hasPausedStateUpdates) {
        throw new Error('[virtual-scroller] State updates have already been paused');
      }

      this.hasPausedStateUpdates = true;
    }
  }, {
    key: "resumeStateUpdates",
    value: function resumeStateUpdates() {
      if (this.pausedStateUpdate) {
        this.pausedStateUpdateAction(this.pausedStateUpdate);
        this.pausedStateUpdate = undefined;
        this.pausedStateUpdateAction = undefined;
      }

      this.hasPausedStateUpdates = false;
    }
  }, {
    key: "verifyState",
    value: function verifyState(expectedState) {
      // `mocha`/`chai`.
      this.virtualScroller.getState().should.deep.include(expectedState);
    }
  }, {
    key: "expectStateUpdate",
    value: function expectStateUpdate(stateUpdate, callback) {
      this.expectedStateUpdates.push({
        stateUpdate: stateUpdate,
        callback: callback
      });
    }
  }, {
    key: "getFirstNonMeasuredItemIndex",
    value: function getFirstNonMeasuredItemIndex() {
      return this.virtualScroller.firstNonMeasuredItemIndex;
    }
  }, {
    key: "getScrollY",
    value: function getScrollY() {
      return this.virtualScroller.scrollableContainer.getScrollY();
    }
  }, {
    key: "scrollTo",
    value: function scrollTo(scrollPosition) {
      this.virtualScroller.scrollableContainer.scrollToY(scrollPosition);
    }
  }, {
    key: "resize",
    value: function resize(_ref2) {
      var scrollableContainerWidth = _ref2.screenWidth,
          scrollableContainerHeight = _ref2.screenHeight,
          columnsCount = _ref2.columnsCount;
      // Resize scrollable container.
      this.setScrollableContainerWidth(scrollableContainerWidth);
      this.setScrollableContainerHeight(scrollableContainerHeight); // Update columns count.

      this.__columnsCount = columnsCount;
    } // Returns a `Promise`.

  }, {
    key: "triggerResize",
    value: function triggerResize(_ref3) {
      var screenWidth = _ref3.screenWidth,
          screenHeight = _ref3.screenHeight,
          columnsCount = _ref3.columnsCount,
          verticalSpacing = _ref3.verticalSpacing;
      this.resize({
        screenWidth: screenWidth,
        screenHeight: screenHeight,
        columnsCount: columnsCount,
        verticalSpacing: verticalSpacing
      }); // Call "on resize" listener.

      return this.virtualScroller.scrollableContainer._triggerResizeListener();
    }
  }, {
    key: "stop",
    value: function stop() {
      if (this.expectedStateUpdates.length > 0) {
        throw new Error("[virtual-scroller] Expected ".concat(this.expectedStateUpdates.length, " state updates which didn't happen:\n").concat(JSON.stringify(this.expectedStateUpdates, null, 2)));
      }

      if (this.pausedStateUpdate) {
        throw new Error('[virtual-scroller] State updates were paused and haven\'t been resumed afterwards');
      }

      this.virtualScroller.stop();
    }
  }, {
    key: "start",
    value: function start() {
      this.virtualScroller.start();
    }
  }, {
    key: "getState",
    value: function getState() {
      return this.virtualScroller.getState();
    }
  }, {
    key: "updateLayout",
    value: function updateLayout() {
      this.virtualScroller.updateLayout();
    }
  }, {
    key: "getItemScrollPosition",
    value: function getItemScrollPosition(i) {
      return this.virtualScroller.getItemScrollPosition(i);
    }
  }, {
    key: "getAverageItemHeight",
    value: function getAverageItemHeight() {
      return this.virtualScroller.itemHeights.getAverage();
    }
  }, {
    key: "setItems",
    value: function setItems(newItems, options) {
      this.virtualScroller.setItems(newItems, options);
    }
  }]);

  return TestVirtualScroller;
}();

export { TestVirtualScroller as default };
//# sourceMappingURL=VirtualScroller.js.map